# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import bcolz
import pandas as pd

from ..utils.py2 import lru_cache


class DividendStore(object):
    def __init__(self, f):
        self._table = bcolz.open(f, 'r')
        self._index = self._table.attrs['line_map']

    @lru_cache(128)
    def get_dividend(self, order_book_id):
        try:
            s, e = self._index[order_book_id]
        except KeyError:
            return pd.DataFrame()

        dividends = self._table[s:e]
        return pd.DataFrame({
            'book_closure_date': pd.Index(pd.Timestamp(str(d)) for d in dividends['closure_date']),
            'ex_dividend_date': pd.Index(pd.Timestamp(str(d)) for d in dividends['ex_date']),
            'payable_date': pd.Index(pd.Timestamp(str(d)) for d in dividends['payable_date']),
            'dividend_cash_before_tax': dividends['cash_before_tax'] / 10000.0,
            'round_lot': dividends['round_lot']
        }, index=pd.Index(pd.Timestamp(str(d)) for d in dividends['announcement_date']))



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time
from .datetime_func import TimeRange
from ..const import COMMISSION_TYPE, MARGIN_TYPE

NIGHT_UNDERLYING_SYMBOL = ["CU", "AL", "ZN", "PB", "SN", "NI", "RB", "HC", "BU", "RU", "AU", "AG", "Y", "M", "A", "B",
                           "P", "J", "JM", "I", "CF", "SR", "OI", "MA", "ZC", "FG", "RM"]

TRADING_PERIOD_DICT = dict()

STOCK_TRADING_PERIOD = [
    TimeRange(start=time(9, 31), end=time(11, 30)),
    TimeRange(start=time(13, 1), end=time(15, 0)),
]

# | 商品期货 WR, FU, CS, C, L, V, PP, BB, FB, JD, WH, PM, RI, SF, SM, RS, JR, LR  | 09:01~10:15, 10:31~11:30, 13:31~15:00 |
time_period1 = [
    TimeRange(start=time(9, 1), end=time(10, 15)),
    TimeRange(start=time(10, 31), end=time(11, 30)),
    TimeRange(start=time(13, 31), end=time(15, 0)),
]
TRADING_PERIOD_DICT.update({underlying_symbol: time_period1 for underlying_symbol in
                       ["WR", "FU", "CS", "C", "L", "V", "PP", "BB", "FB", "JD", "WH", "PM", "RI", "SF", "SM", "RS",
                        "JR", "LR"]})

# | 商品期货 Y, M, A, B, P, J, JM, I, CF, SR, OI, TA, MA, ZC, FG, RM | 21:01~23:30, 09:01~10:15, 10:31~11:30, 13:31~15:00 |
time_period2 = [
    TimeRange(start=time(21, 1), end=time(23, 30)),
    TimeRange(start=time(9, 1), end=time(10, 15)),
    TimeRange(start=time(10, 31), end=time(11, 30)),
    TimeRange(start=time(13, 31), end=time(15, 0)),
]
TRADING_PERIOD_DICT.update({underlying_symbol: time_period2 for underlying_symbol in
                       ["Y", "M", "A", "B", "P", "J", "JM", "I", "CF", "SR", "OI", "TA", "MA", "ZC", "FG", "RM"]})

# | 商品期货 CU, AL, ZN, PB, SN, NI | 21:01~1:00, 09:01~10:15, 10:31~11:30, 13:31~15:00 |
time_period3 = [
    TimeRange(start=time(21, 1), end=time(23, 59)),
    TimeRange(start=time(0, 0), end=time(1, 0)),
    TimeRange(start=time(9, 1), end=time(10, 15)),
    TimeRange(start=time(10, 31), end=time(11, 30)),
    TimeRange(start=time(13, 31), end=time(15, 0)),
]
TRADING_PERIOD_DICT.update({underlying_symbol: time_period3 for underlying_symbol in ["CU", "AL", "ZN", "PB", "SN", "NI"]})

# | 商品期货 RB, HC, BU, RU | 21:01~23:00, 09:01~10:15, 10:31~11:30, 13:31~15:00 |
time_period4 = [
    TimeRange(start=time(21, 1), end=time(23, 0)),
    TimeRange(start=time(9, 1), end=time(10, 15)),
    TimeRange(start=time(10, 31), end=time(11, 30)),
    TimeRange(start=time(13, 31), end=time(15, 0)),
]
TRADING_PERIOD_DICT.update({underlying_symbol: time_period4 for underlying_symbol in ["RB", "HC", "BU", "RU"]})

# | 商品期货 AU, AG | 21:01~2:30, 09:01~10:15, 10:31~11:30, 13:31~15:00 |
time_period5 = [
    TimeRange(start=time(21, 1), end=time(23, 59)),
    TimeRange(start=time(0, 0), end=time(2, 30)),
    TimeRange(start=time(9, 1), end=time(10, 15)),
    TimeRange(start=time(10, 31), end=time(11, 30)),
    TimeRange(start=time(13, 31), end=time(15, 0)),
]
TRADING_PERIOD_DICT.update({underlying_symbol: time_period5 for underlying_symbol in ["AU", "AG"]})

# | 股指期货 product='Index' | 09:31~11:30, 13:01~15:00 |
time_period6 = [
    TimeRange(start=time(9, 31), end=time(11, 30)),
    TimeRange(start=time(13, 1), end=time(15, 0)),
]
TRADING_PERIOD_DICT.update({underlying_symbol: time_period6 for underlying_symbol in ["IF", "IH", "IC"]})

# | 国债期货 product='Government' | 09:16~11:30, 13:01~15:15|
time_period7 = [
    TimeRange(start=time(9, 16), end=time(11, 30)),
    TimeRange(start=time(13, 1), end=time(15, 15)),
]
TRADING_PERIOD_DICT.update({underlying_symbol: time_period7 for underlying_symbol in ["T", "TF"]})




# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import json

from dateutil.parser import parse

from ..execution_context import ExecutionContext
from ..environment import Environment
from ..utils.exception import patch_user_exc, ModifyExceptionFromType
from ..const import EXC_TYPE, EXECUTION_PHASE
from ..events import EVENT

try:
    from inspect import signature
except ImportError:
    from funcsigs import signature


def market_close(hour=0, minute=0):
    minutes_since_midnight = 15 * 60 - hour * 60 - minute
    if minutes_since_midnight < 13 * 60:
        minutes_since_midnight -= 90
    return minutes_since_midnight


def market_open(hour=0, minute=0):
    minutes_since_midnight = 9 * 60 + 31 + hour * 60 + minute
    if minutes_since_midnight > 11 * 60 + 30:
        minutes_since_midnight += 90
    return minutes_since_midnight


_scheduler = None


@ExecutionContext.enforce_phase(EXECUTION_PHASE.ON_INIT)
def run_daily(func, time_rule=None):
    _scheduler.run_daily(func, time_rule)


@ExecutionContext.enforce_phase(EXECUTION_PHASE.ON_INIT)
def run_weekly(func, weekday=None, tradingday=None, time_rule=None):
    _scheduler.run_weekly(func, weekday=weekday, tradingday=tradingday, time_rule=time_rule)


@ExecutionContext.enforce_phase(EXECUTION_PHASE.ON_INIT)
def run_monthly(func, tradingday=None, time_rule=None, **kwargs):
    _scheduler.run_monthly(func, tradingday=tradingday, time_rule=time_rule, **kwargs)


def _verify_function(name, func):
    if not callable(func):
        raise patch_user_exc(ValueError('scheduler.{}: func should be callable'.format(name)))
    sig = signature(func)
    if len(sig.parameters) != 2:
        raise patch_user_exc(TypeError(
            'scheduler.{}: func should take exactly 2 arguments (context, bar_dict)'.format(name)))


class Scheduler(object):
    _TRADING_DATES = None

    @classmethod
    def set_trading_dates_(cls, trading_dates):
        cls._TRADING_DATES = trading_dates

    def __init__(self, frequency):
        self._registry = []
        self._today = None
        self._this_week = None
        self._this_month = None
        self._last_minute = 0
        self._current_minute = 0
        self._stage = None
        self._ucontext = None
        self._frequency = frequency

        event_bus = Environment.get_instance().event_bus
        event_bus.add_listener(EVENT.PRE_BEFORE_TRADING, self.next_day_)
        event_bus.add_listener(EVENT.BEFORE_TRADING, self.before_trading_)
        event_bus.add_listener(EVENT.BAR, self.next_bar_)

    def set_user_context(self, ucontext):
        self._ucontext = ucontext

    @staticmethod
    def _always_true():
        return True

    def _is_weekday(self, wd):
        return self._today.weekday() == wd

    def _is_nth_trading_day_in_week(self, n):
        try:
            return self._this_week[n] == self._today
        except IndexError:
            return False

    def _is_nth_trading_day_in_month(self, n):
        try:
            return self._this_month[n] == self._today
        except IndexError:
            return False

    def _should_trigger(self, n):
        # 非股票交易时间段不触发
        if self._current_minute < 9*60+31 or self._current_minute > 15*60:
            return False

        return self._last_minute < n <= self._current_minute

    def _is_before_trading(self):
        return self._stage == 'before_trading'

    def _time_rule_for(self, time_rule):
        if time_rule == 'before_trading':
            return lambda: self._is_before_trading()

        time_rule = time_rule if time_rule else self._minutes_since_midnight(9, 31)
        return lambda: self._should_trigger(time_rule)

    def run_daily(self, func, time_rule=None):
        _verify_function('run_daily', func)
        self._registry.append((self._always_true,
                               self._time_rule_for(time_rule),
                               func))

    def run_weekly(self, func, weekday=None, tradingday=None, time_rule=None):
        _verify_function('run_weekly', func)
        if (weekday is not None and tradingday is not None) or (weekday is None and tradingday is None):
            raise patch_user_exc(ValueError('select one of weekday/tradingday'))

        if weekday is not None:
            if weekday < 1 or weekday > 7:
                raise patch_user_exc(ValueError('invalid weekday, should be in [1, 7]'))
            day_checker = lambda: self._is_weekday(weekday - 1)
        else:
            if tradingday > 5 or tradingday < -5 or tradingday == 0:
                raise patch_user_exc(ValueError('invalid trading day, should be in [-5, 0), (0, 5]'))
            if tradingday > 0:
                tradingday -= 1
            day_checker = lambda: self._is_nth_trading_day_in_week(tradingday)

        time_checker = self._time_rule_for(time_rule)

        self._registry.append((day_checker, time_checker, func))

    def run_monthly(self, func, tradingday=None, time_rule=None, **kwargs):
        _verify_function('run_monthly', func)
        if tradingday is None and 'monthday' in kwargs:
            tradingday = kwargs.pop('monthday')
        if kwargs:
            raise patch_user_exc(ValueError('unknown argument: {}'.format(kwargs)))

        if tradingday is None:
            raise patch_user_exc(ValueError('tradingday is required'))

        if tradingday > 23 or tradingday < -23 or tradingday == 0:
            raise patch_user_exc(ValueError('invalid tradingday, should be in [-23, 0), (0, 23]'))
        if tradingday > 0:
            tradingday -= 1

        time_checker = self._time_rule_for(time_rule)

        self._registry.append((lambda: self._is_nth_trading_day_in_month(tradingday),
                               time_checker, func))

    def next_day_(self, event):
        if len(self._registry) == 0:
            return

        self._today = Environment.get_instance().trading_dt.date()
        self._last_minute = 0
        self._current_minute = 0
        if not self._this_week or self._today > self._this_week[-1]:
            self._fill_week()
        if not self._this_month or self._today > self._this_month[-1]:
            self._fill_month()

    @staticmethod
    def _minutes_since_midnight(hour, minute):
        return hour * 60 + minute

    def next_bar_(self, event):
        bars = event.bar_dict
        with ExecutionContext(EXECUTION_PHASE.SCHEDULED):
            self._current_minute = self._minutes_since_midnight(self._ucontext.now.hour, self._ucontext.now.minute)
            for day_rule, time_rule, func in self._registry:
                if day_rule() and time_rule():
                    with ModifyExceptionFromType(EXC_TYPE.USER_EXC):
                        func(self._ucontext, bars)
            self._last_minute = self._current_minute

    def before_trading_(self, event):
        with ExecutionContext(EXECUTION_PHASE.BEFORE_TRADING):
            self._stage = 'before_trading'
            for day_rule, time_rule, func in self._registry:
                if day_rule() and time_rule():
                    with ModifyExceptionFromType(EXC_TYPE.USER_EXC):
                        func(self._ucontext, None)
            self._stage = None

    def _fill_week(self):
        weekday = self._today.isoweekday()
        weekend = self._today + datetime.timedelta(days=7-weekday)
        week_start = weekend - datetime.timedelta(days=6)

        left = self._TRADING_DATES.searchsorted(week_start)
        right = self._TRADING_DATES.searchsorted(weekend, side='right')
        self._this_week = [d.date() for d in self._TRADING_DATES[left:right]]

    def _fill_month(self):
        try:
            month_end = self._today.replace(month=self._today.month+1, day=1)
        except ValueError:
            month_end = self._today.replace(year=self._today.year+1, month=1, day=1)

        month_begin = self._today.replace(day=1)
        left, right = self._TRADING_DATES.searchsorted(month_begin), self._TRADING_DATES.searchsorted(month_end)
        self._this_month = [d.date() for d in self._TRADING_DATES[left:right]]

    def set_state(self, state):
        r = json.loads(state.decode('utf-8'))
        self._today = parse(r['today']).date()
        self._last_minute = r['last_minute']
        self._fill_month()
        self._fill_week()

    def get_state(self):
        if self._today is None:
            return None

        return json.dumps({
            'today': self._today.strftime('%Y-%m-%d'),
            'last_minute': self._last_minute
        }).encode('utf-8')



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .base_position import BasePosition
from .future_position import FuturePosition
from .stock_position import StockPosition


class Positions(dict):
    def __init__(self, position_cls):
        super(Positions, self).__init__()
        self._position_cls = position_cls
        self._cached_positions = {}

    def __missing__(self, key):
        if key not in self._cached_positions:
            self._cached_positions[key] = self._position_cls(key)
        return self._cached_positions[key]

    def get_or_create(self, key):
        if key not in self:
            self[key] = self._position_cls(key)
        return self[key]



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rqalpha.interface import AbstractBroker
from rqalpha.utils.logger import user_system_log
from rqalpha.utils.i18n import gettext as _
from rqalpha.events import EVENT, Event
from rqalpha.model.order import LimitOrder
from rqalpha.model.trade import Trade
from rqalpha.const import BAR_STATUS, SIDE
from rqalpha.environment import Environment

from .decider import CommissionDecider, SlippageDecider, TaxDecider
from .utils import init_portfolio


class SignalBroker(AbstractBroker):
    def __init__(self, env, mod_config):
        self._env = env
        self._commission_decider = CommissionDecider(mod_config.commission_multiplier)
        self._slippage_decider = SlippageDecider(mod_config.slppage)
        self._tax_decider = TaxDecider()

    def get_portfolio(self):
        return init_portfolio(self._env)

    def get_open_orders(self, order_book_id=None):
        return []

    def submit_order(self, order):
        account = Environment.get_instance().get_account(order.order_book_id)
        self._env.event_bus.publish_event(Event(EVENT.ORDER_PENDING_NEW, account=account, order=order))
        if order.is_final():
            return
        order.active()
        self._env.event_bus.publish_event(Event(EVENT.ORDER_CREATION_PASS, account=account, order=order))
        self._match(account, order)

    def cancel_order(self, order):
        user_system_log.error(_(u"cancel_order function is not supported in signal mode"))
        return None

    def _match(self, account, order):
        # TODO support tick cal
        env = Environment.get_instance()
        bar = env.get_bar(order.order_book_id)
        bar_status = bar._bar_status

        if bar_status == BAR_STATUS.ERROR:
            listed_date = bar.instrument.listed_date.date()
            if listed_date == self._trading_dt.date():
                reason = _(
                    "Order Cancelled: current security [{order_book_id}] can not be traded in listed date [{listed_date}]").format(
                    order_book_id=order.order_book_id,
                    listed_date=listed_date,
                )
            else:
                reason = _(u"Order Cancelled: current bar [{order_book_id}] miss market data.").format(
                    order_book_id=order.order_book_id)
            order.mark_rejected(reason)
            self._env.event_bus.publish_event(Event(EVENT.ORDER_UNSOLICITED_UPDATE, account=account, order=order))
            return

        if isinstance(order.style, LimitOrder):
            deal_price = order.style.get_limit_price()
        else:
            deal_price = bar.close

        deal_price = min(deal_price, bar.high)
        deal_price = max(deal_price, bar.low)

        deal_price = self._slippage_decider.get_trade_price(order.side, deal_price)

        if (order.side == SIDE.BUY and bar_status == BAR_STATUS.LIMIT_UP) or (
                order.side == SIDE.SELL and bar_status == BAR_STATUS.LIMIT_DOWN):
            user_system_log.warning(_(u"You have traded {order_book_id} with {quantity} lots in {bar_status}").format(
                order_book_id=order.order_book_id,
                quantity=order.quantity,
                bar_status=bar_status
            ))
        ct_amount = account.portfolio.positions.get_or_create(order.order_book_id).cal_close_today_amount(order.quantity, order.side)
        trade = Trade.__from_create__(
            order_id=order.order_id,
            calendar_dt=self._calendar_dt,
            trading_dt=self._trading_dt,
            price=deal_price,
            amount=order.quantity,
            side=order.side,
            position_effect=order.position_effect,
            order_book_id=order.order_book_id,
            frozen_price=order.frozen_price,
            close_today_amount=ct_amount
        )
        trade._commission = self._commission_decider.get_commission(account.type, trade)
        trade._tax = self._tax_decider.get_tax(account.type, trade)
        order.fill(trade)

        env.event_bus.publish_event(Event(EVENT.TRADE, account=account, trade=trade))



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from ..interface import AbstractPriceBoard
from ..environment import Environment
from ..events import EVENT
from ..const import ACCOUNT_TYPE


class BarDictPriceBoard(AbstractPriceBoard):
    def __init__(self, bar_dict):
        self._bar_dict = bar_dict
        self._settlement_lock = False
        self._env = Environment.get_instance()
        if ACCOUNT_TYPE.FUTURE in self._env.config.base.account_list:
            self._env.event_bus.add_listener(EVENT.PRE_SETTLEMENT, self._lock_settlement)
            self._env.event_bus.add_listener(EVENT.POST_BEFORE_TRADING, self._unlock_settlement)

    def get_last_price(self, order_book_id):
        if self._settlement_lock:
            return self._env.data_proxy.get_settle_price(order_book_id, self._env.trading_dt)
        else:
            return self._bar_dict[order_book_id].last

    def get_limit_up(self, order_book_id):
        return self._bar_dict[order_book_id].limit_up

    def get_limit_down(self, order_book_id):
        return self._bar_dict[order_book_id].limit_down

    def _lock_settlement(self, event):
        self._settlement_lock = True

    def _unlock_settlement(self, event):
        self._settlement_lock = False



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect


# Unused Cached Class
class CachedProperty(object):

    def __init__(self, func):
        self.func = func

    def __get__(self, obj, cls):
        if obj is None:
            return self
        if "__cached__" not in obj.__dict__:
            obj.__dict__["__cached__"] = []
        value = obj.__dict__[self.func.__name__] = self.func(obj)
        obj.__dict__["__cached__"].append(self.func.__name__)
        return value

    @property
    def annotation(self):
        return inspect.signature(self.func).return_annotation


def clear_cache(obj):
    if "__cached__" not in obj.__dict__:
        return
    for cached_key in obj.__dict__["__cached__"]:
        obj.__dict__.pop(cached_key)
    obj.__dict__["__cached__"] = []


# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import division

import numpy as np

APPROX_BDAYS_PER_MONTH = 21
APPROX_BDAYS_PER_YEAR = 244

MONTHS_PER_YEAR = 12
WEEKS_PER_YEAR = 52

DAILY = 'daily'
WEEKLY = 'weekly'
MONTHLY = 'monthly'
YEARLY = 'yearly'

ANNUALIZATION_FACTORS = {
    DAILY: APPROX_BDAYS_PER_YEAR,
    WEEKLY: WEEKS_PER_YEAR,
    MONTHLY: MONTHS_PER_YEAR
}


def _annual_factor(period):
    try:
        return ANNUALIZATION_FACTORS[period]
    except KeyError:
        raise ValueError("period cannot be {}, possible values: {}".format(
            period, ", ".join(ANNUALIZATION_FACTORS.keys())))


class Risk(object):
    def __init__(self, daily_returns, benchmark_daily_returns, risk_free_rate, days, period=DAILY):
        assert(len(daily_returns) == len(benchmark_daily_returns))

        self._portfolio = daily_returns
        self._benchmark = benchmark_daily_returns
        self._risk_free_rate = risk_free_rate
        self._annual_factor = _annual_factor(period)
        self._daily_risk_free_rate = self._risk_free_rate / self._annual_factor

        self._alpha = None
        self._beta = None
        self._sharpe = None
        self._return = np.expm1(np.log1p(self._portfolio).sum())
        self._annual_return = (1 + self._return) ** (365 / days) - 1
        self._benchmark_return = np.expm1(np.log1p(self._benchmark).sum())
        self._benchmark_annual_return = (1 + self._benchmark_return) ** (365 / days) - 1
        self._max_drawdown = None
        self._volatility = None
        self._annual_volatility = None
        self._benchmark_volatility = None
        self._benchmark_annual_volatility = None
        self._information_ratio = None
        self._sortino = None
        self._tracking_error = None
        self._annual_tracking_error = None
        self._downside_risk = None
        self._annual_downside_risk = None
        self._calmar = None
        self._avg_excess_return = None

    @property
    def return_rate(self):
        return self._return

    @property
    def annual_return(self):
        return self._annual_return

    @property
    def benchmark_return(self):
        return self._benchmark_return

    @property
    def benchmark_annual_return(self):
        return self._benchmark_annual_return

    @property
    def alpha(self):
        if self._alpha is not None:
            return self._alpha

        if len(self._portfolio) < 2:
            self._alpha = np.nan
            self._beta = np.nan
            return np.nan

        self._alpha = np.mean(self._portfolio - self._daily_risk_free_rate + self.beta * (
                    self._benchmark - self._daily_risk_free_rate
                )) * self._annual_factor
        return self._alpha

    @property
    def beta(self):
        if self._beta is not None:
            return self._beta

        if len(self._portfolio) < 2:
            self._beta = np.nan
            return self._beta

        cov = np.cov(np.vstack([
            self._portfolio,
            self._benchmark
        ]), ddof=1)
        self._beta = cov[0][1] / cov[1][1]
        return self._beta

    @property
    def avg_excess_return(self):
        if self._avg_excess_return is not None:
            return self._avg_excess_return
        #self._avg_excess_return = 1.0 / len(self._portfolio) * (self._portfolio - self._daily_risk_free_rate).sum()
        self._avg_excess_return = np.mean(self._portfolio - self._daily_risk_free_rate)
        return self._avg_excess_return

    def _calc_volatility(self):
        if len(self._portfolio) < 2:
            self._volatility = 0
            self._annual_volatility = 0
        else:
            #std = self._portfolio.std(ddof=1)
            self._volatility = self._portfolio.std(ddof=1)
            self._annual_volatility = self._volatility * (self._annual_factor ** 0.5)

    @property
    def volatility(self):
        if self._volatility is not None:
            return self._volatility

        self._calc_volatility()
        return self._volatility

    @property
    def annual_volatility(self):
        if self._annual_volatility is not None:
            return self._annual_volatility

        self._calc_volatility()
        return self._annual_volatility

    def _calc_benchmark_volatility(self):
        if len(self._benchmark) < 2:
            self._benchmark_volatility = 0
            self._benchmark_annual_volatility = 0
        else:
            #std = self._benchmark.std(ddof=1)
            self._benchmark_volatility = self._benchmark.std(ddof=1)
            self._benchmark_annual_volatility = self._benchmark_volatility * (self._annual_factor ** 0.5)

    @property
    def benchmark_volatility(self):
        if self._benchmark_volatility is not None:
            return self._benchmark_volatility

        self._calc_benchmark_volatility()
        return self._benchmark_volatility

    @property
    def benchmark_annual_volatility(self):
        if self._benchmark_annual_volatility is not None:
            return self._benchmark_annual_volatility

        self._calc_benchmark_volatility()
        return self._benchmark_annual_volatility

    @property
    def max_drawdown(self):
        if self._max_drawdown is not None:
            return self._max_drawdown

        if len(self._portfolio) < 1:
            self._max_drawdown = np.nan
            return np.nan

        df_cum = np.exp(np.log1p(self._portfolio).cumsum())
        max_return = np.maximum.accumulate(df_cum)
        self._max_drawdown = ((df_cum - max_return) / max_return).min()
        return abs(self._max_drawdown)

    def _calc_tracking_error(self):
        if len(self._portfolio) < 2:
            self._tracking_error = 0
            self._annual_tracking_error = 0
            return 0

        active_return = self._portfolio - self._benchmark
        #sum_mean_squares = np.sum(np.square(active_return))
        #self._avg_tracking_return = np.mean(np.sum(active_return))
        self._avg_tracking_return = np.mean(active_return)
        #self._tracking_error = (sum_mean_squares ** 0.5) * ((self._annual_factor / (len(active_return) - 1)) ** 0.5)
        self._tracking_error = active_return.std(ddof=1)
        #self._annual_tracking_error = (sum_mean_squares ** 0.5) / (self._annual_factor ** 0.5)
        self._annual_tracking_error = self._tracking_error * (self._annual_factor ** 0.5)

    @property
    def tracking_error(self):
        if self._tracking_error is not None:
            return self._tracking_error

        self._calc_tracking_error()
        return self._tracking_error

    @property
    def annual_tracking_error(self):
        if self._annual_tracking_error is not None:
            return self._annual_tracking_error

        self._calc_tracking_error()
        return self._annual_tracking_error

    @property
    def information_ratio(self):
        if self._information_ratio is not None:
            return self._information_ratio

        if len(self._portfolio) < 2:
            self._information_ratio = np.nan
            return np.nan

        if self.tracking_error == 0:
            self._information_ratio = np.nan
            return np.nan

        self._information_ratio = np.sqrt(self._annual_factor) * self._avg_tracking_return / self.tracking_error
        return self._information_ratio

    @property
    def sharpe(self):
        if self._sharpe is not None:
            return self._sharpe

        if self.volatility == 0:
            self._sharpe = np.nan
            return np.nan

        std_excess_return = np.sqrt((1 / (len(self._portfolio) - 1)) * np.sum(
            (self._portfolio - self._daily_risk_free_rate - self.avg_excess_return) ** 2
        ))
        self._sharpe = np.sqrt(self._annual_factor) * self.avg_excess_return / std_excess_return
        return self._sharpe

    def _calc_downside_risk(self):
        if len(self._portfolio) < 2:
            self._annual_downside_risk = 0
            self._downside_risk = 0
            return 0
        diff = self._portfolio - self._benchmark
        diff[diff > 0] = 0
        sum_mean_squares = np.sum(np.square(diff))
        #self._annual_downside_risk = (sum_mean_squares ** 0.5) * \
        #                             ((self._annual_factor / (len(self._portfolio) - 1)) ** 0.5)
        self._downside_risk = (sum_mean_squares / (len(diff) - 1)) ** 0.5
        self._annual_downside_risk = self._downside_risk * (self._annual_factor ** 0.5)

    @property
    def downside_risk(self):
        if self._downside_risk is not None:
            return self._downside_risk

        self._calc_downside_risk()
        return self._downside_risk

    @property
    def annual_downside_risk(self):
        if self._annual_downside_risk is not None:
            return self._annual_downside_risk

        self._calc_downside_risk()
        return self._annual_downside_risk

    @property
    def sortino(self):
        if self._sortino is not None:
            return self._sortino

        if self.downside_risk == 0:
            self._sortino = np.nan
            return np.nan

        self._sortino = self._annual_factor * self.avg_excess_return / self.annual_downside_risk
        return self._sortino

    @property
    def calmar(self):
        if self._calmar is not None:
            return self._calmar

        max_dd = self.max_drawdown
        if max_dd < 0:
            tmp = self._annual_return / -max_dd
            if np.isinf(tmp):
                self._calmar = np.nan
            else:
                self._calmar = tmp
        else:
            self._calmar = np.nan

        return self._calmar

    def all(self):
        result = {
            'return': self.return_rate,
            'annual_return': self.annual_return,
            'benchmark_return': self.benchmark_return,
            'benchmark_annual_return': self.benchmark_annual_return,
            'alpha': self.alpha,
            'beta': self.beta,
            'sharpe': self.sharpe,
            'max_drawdown': self.max_drawdown,
            'volatility': self.volatility,
            'annual_volatility': self.annual_volatility,
            'information_ratio': self.information_ratio,
            'downside_risk': self.downside_risk,
            'sortino': self.sortino,
            'tracking_error': self.tracking_error,
            'calmar': self.calmar,
        }

        # now all are done, _portfolio, _benchmark not needed now
        self._portfolio = None
        self._benchmark = None
        return result



#!/usr/bin/env python
# encoding: utf-8
import inspect

def test_get_order():
    from rqalpha.api import order_shares, get_order
    def init(context):
        context.s1 = '000001.XSHE'
        context.amount = 100

    def handle_bar(context, bar_dict):
        order_id = order_shares(context.s1, context.amount, style=LimitOrder(9.5))
        order = get_order(order_id)
        assert order.order_book_id == context.s1
        assert order.quantity == context.amount
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
test_get_order_code_new = "".join(inspect.getsourcelines(test_get_order)[0])


def test_get_open_order():
    from rqalpha.api import order_shares, get_open_orders, get_order
    def init(context):
        context.s1 = '000001.XSHE'
        context.limitprice = 8.9
        context.amount = 100
        context.counter = 0
        context.order_id = None

    def handle_bar(context, bar_dict):
        context.counter += 1

        order = order_shares(context.s1, context.amount, style=LimitOrder(context.limitprice))
        context.order_id = order.order_id
        print('cash: ', context.portfolio.cash)
        print('check_get_open_orders done')
        print(order.order_id)
        print(get_open_orders().keys())
        print(get_open_orders())
        print(get_order(order.order_id))
        if context.counter == 2:
            assert order.order_id in get_open_orders()
        context.counter = 0
test_get_open_order_code_new = "".join(inspect.getsourcelines(test_get_open_order)[0])


def test_cancel_order():
    from rqalpha.api import order_shares, cancel_order, get_order
    def init(context):
        context.s1 = '000001.XSHE'
        context.limitprice = 8.59
        context.amount = 100

    def handle_bar(context, bar_dict):
        order_id = order_shares(context.s1, context.amount, style=LimitOrder(context.limitprice))
        cancel_order(order_id)
        order = get_order(order_id)
        assert order.order_book_id == context.s1
        assert order.filled_quantity == 0
        return order_id
        assert order.price == context.limitprice
test_cancel_order_code_new = "".join(inspect.getsourcelines(test_cancel_order)[0])


def test_update_universe():
    from rqalpha.api import update_universe, history_bars
    def init(context):
        context.s1 = '000001.XSHE'
        context.s2 = '600340.XSHG'
        context.order_count = 0
        context.amount = 100

    def handle_bar(context, bar_dict):
        context.order_count += 1
        if context.order_count == 1:
            update_universe(context.s2)
            his = history_bars(context.s2, 5, '1d', 'close')
            print(sorted(his.tolist()))
            print(sorted([24.1, 23.71, 23.82, 23.93, 23.66]))
            assert sorted(his.tolist()) == sorted([26.06, 26.13, 26.54, 26.6, 26.86])
test_update_universe_code_new = "".join(inspect.getsourcelines(test_update_universe)[0])


def test_subscribe():
    from rqalpha.api import subscribe
    def init(context):
        context.f1 = 'AU88'
        context.amount = 1
        subscribe(context.f1)

    def handle_bar(context, bar_dict):
        assert context.f1 in context.universe
test_subscribe_code_new = "".join(inspect.getsourcelines(test_subscribe)[0])


def test_unsubscribe():
    from rqalpha.api import subscribe, unsubscribe
    def init(context):
        context.f1 = 'AU88'
        context.amount = 1
        subscribe(context.f1)
        unsubscribe(context.f1)

    def handle_bar(context, bar_dict):
        assert context.f1 not in context.universe
test_unsubscribe_code_new = "".join(inspect.getsourcelines(test_unsubscribe)[0])


def test_get_yield_curve():
    from rqalpha.api import get_yield_curve
    def init(context):
        pass

    def handle_bar(context, bar_dict):
        df = get_yield_curve('20161101')
        assert df.iloc[0, 0] == 0.019923
        assert df.iloc[0, 6] == 0.021741
test_get_yield_curve_code_new = "".join(inspect.getsourcelines(test_get_yield_curve)[0])


def test_history_bars():
    from rqalpha.api import history_bars
    def init(context):
        context.s1 = '000001.XSHE'
        pass

    def handle_bar(context, bar_dict):
        return_list = history_bars(context.s1, 5, '1d', 'close')
        if str(context.now.date()) == '2016-12-29':
            assert return_list.tolist() == [9.08, 9.1199, 9.08, 9.06, 9.08]
test_history_bars_code_new = "".join(inspect.getsourcelines(test_history_bars)[0])


def test_all_instruments():
    from rqalpha.api import all_instruments
    def init(context):
        pass

    def handle_bar(context, bar_dict):
        df = all_instruments('FenjiA')
        df_to_assert = df.loc[df['order_book_id'] == '150247.XSHE']
        assert df_to_assert.iloc[0, 0] == 'CMAJ'
        assert df_to_assert.iloc[0, 7] == '工银中证传媒A'
        assert all_instruments().shape >= (8000, 4)
        assert all_instruments('CS').shape >= (3000, 16)
        assert all_instruments('ETF').shape >= (120, 9)
        assert all_instruments('LOF').shape >= (130, 9)
        assert all_instruments('FenjiMu').shape >= (10, 9)
        assert all_instruments('FenjiA').shape >= (120, 9)
        assert all_instruments('FenjiB').shape >= (140, 9)
        assert all_instruments('INDX').shape >= (500, 8)
        assert all_instruments('Future').shape >= (3500, 16)
test_all_instruments_code_new = "".join(inspect.getsourcelines(test_all_instruments)[0])

def test_instruments_code():
    from rqalpha.api import instruments
    def init(context):
        context.s1 = '000001.XSHE'
        pass

    def handle_bar(context, bar_dict):
        print('hello')
        ins = instruments(context.s1)
        assert ins.sector_code_name == '金融'
        assert ins.symbol == '平安银行'
        assert ins.order_book_id == context.s1
        assert ins.type == 'CS'
        print('world')
test_instruments_code_new = "".join(inspect.getsourcelines(test_instruments_code)[0])


def test_sector():
    from rqalpha.api import sector
    def init(context):
        pass

    def handle_bar(context, bar_dict):
        assert len(sector('金融')) >= 180
test_sector_code_new = "".join(inspect.getsourcelines(test_sector)[0])


def test_industry():
    from rqalpha.api import industry, instruments
    def init(context):
        context.s1 = '000001.XSHE'
        context.s2 = '600340.XSHG'

    def handle_bar(context, bar_dict):
        ins_1 = instruments(context.s1)
        ins_2 = instruments(context.s2)
        industry_list_1 = industry(ins_1.industry_name)
        industry_list_2 = industry(ins_2.industry_name)
        assert context.s1 in industry_list_1
        assert context.s2 in industry_list_2
test_industry_code_new = "".join(inspect.getsourcelines(test_industry)[0])


def test_concept():
    from rqalpha.api import concept, instruments
    def init(context):
        context.s1 = '000002.XSHE'

    def handle_bar(context, bar_dict):
        ins = instruments(context.s1)
        concept_list = concept(ins.concept_names[:4])  # 取 concept_names 的前4个字，即 context.s1 的第一个概念
        assert context.s1 in concept_list
        assert len(concept_list) >= 90
test_concept_code_new = "".join(inspect.getsourcelines(test_concept)[0])


def test_get_trading_dates():
    from rqalpha.api import get_trading_dates
    import datetime
    def init(context):
        pass

    def handle_bar(context, bar_dict):
        trading_dates_list = get_trading_dates('2016-12-15', '2017-01-03')
        correct_dates_list = [datetime.date(2016, 12, 15), datetime.date(2016, 12, 16), datetime.date(2016, 12, 19),
                              datetime.date(2016, 12, 20), datetime.date(2016, 12, 21), datetime.date(2016, 12, 22),
                              datetime.date(2016, 12, 23), datetime.date(2016, 12, 26), datetime.date(2016, 12, 27),
                              datetime.date(2016, 12, 28), datetime.date(2016, 12, 29), datetime.date(2016, 12, 30),
                              datetime.date(2017, 1, 3)]
        assert sorted([item.strftime("%Y%m%d") for item in correct_dates_list]) == sorted(
            [item.strftime("%Y%m%d") for item
             in trading_dates_list])
test_get_trading_dates_code_new = "".join(inspect.getsourcelines(test_get_trading_dates)[0])


def test_get_previous_trading_date():
    from rqalpha.api import get_previous_trading_date
    def init(context):
        pass

    def handle_bar(context, bar_dict):
        assert str(get_previous_trading_date('2017-01-03').date()) == '2016-12-30'
        assert str(get_previous_trading_date('2016-01-03').date()) == '2015-12-31'
        assert str(get_previous_trading_date('2015-01-03').date()) == '2014-12-31'
        assert str(get_previous_trading_date('2014-01-03').date()) == '2014-01-02'
        assert str(get_previous_trading_date('2010-01-03').date()) == '2009-12-31'
        assert str(get_previous_trading_date('2009-01-03').date()) == '2008-12-31'
        assert str(get_previous_trading_date('2005-01-05').date()) == '2005-01-04'
test_get_previous_trading_date_code_new = "".join(inspect.getsourcelines(test_get_previous_trading_date)[0])


def test_get_next_trading_date():
    from rqalpha.api import get_next_trading_date
    def init(context):
        pass

    def handle_bar(context, bar_dict):
        assert str(get_next_trading_date('2017-01-03').date()) == '2017-01-04'
        assert str(get_next_trading_date('2007-01-03').date()) == '2007-01-04'
test_get_next_trading_date_code_new = "".join(inspect.getsourcelines(test_get_next_trading_date)[0])


def test_get_dividend():
    from rqalpha.api import get_dividend
    import pandas
    def init(context):
        pass

    def handle_bar(context, bar_dict):
        df = get_dividend('000001.XSHE', start_date='20130104')
        df_to_assert = df.loc[df['book_closure_date'] == '	2013-06-19']
        assert df.shape >= (4, 5)
        assert df_to_assert.iloc[0, 1] == 0.9838
        assert df_to_assert.iloc[0, 3] == pandas.tslib.Timestamp('2013-06-20 00:00:00')
test_get_dividend_code_new = "".join(inspect.getsourcelines(test_get_dividend)[0])

# =================== 以下把代码写为纯字符串 ===================

test_get_order_code = '''
from rqalpha.api import order_shares, get_order
def init(context):
    context.s1 = '000001.XSHE'
    context.amount = 100

def handle_bar(context, bar_dict):
    assert 1 == 2
    order_id = order_shares(context.s1, context.amount, style=LimitOrder(9.5))
    order = get_order(order_id)
    assert order.order_book_id == context.s1
    assert order.quantity == context.amount
    assert order.unfilled_quantity + order.filled_quantity == order.quantity

'''

test_get_open_order_code = '''
from rqalpha.api import order_shares, get_open_orders
def init(context):
    context.s1 = '000001.XSHE'
    context.limitprice = 8.9
    context.amount = 100
    context.counter = 0
    context.order_id = None

def handle_bar(context, bar_dict):
    context.counter += 1

    order = order_shares(context.s1, context.amount, style=LimitOrder(context.limitprice))
    context.order_id = order.order_id
    print('cash: ', context.portfolio.cash)
    print('check_get_open_orders done')
    print(order.order_id)
    print(get_open_orders().keys())
    print(get_open_orders())
    print(get_order(order.order_id))
    if context.counter == 2:
        assert order.order_id in get_open_orders()
    context.counter = 0
'''

test_cancel_order_code = '''
from rqalpha.api import order_shares, cancel_order, get_order
def init(context):
    context.s1 = '000001.XSHE'
    context.limitprice = 8.59
    context.amount = 100

def handle_bar(context, bar_dict):
    order_id = order_shares(context.s1, context.amount, style=LimitOrder(context.limitprice))
    cancel_order(order_id)
    order = get_order(order_id)
    assert order.order_book_id == context.s1
    assert order.filled_quantity == 0
    return order_id
    assert order.price == context.limitprice
'''

test_update_universe_code = '''
from rqalpha.api import update_universe, history_bars
def init(context):
    context.s1 = '000001.XSHE'
    context.s2 = '600340.XSHG'
    context.order_count = 0
    context.amount = 100

def handle_bar(context, bar_dict):
    context.order_count += 1
    if context.order_count == 1:
        update_universe(context.s2)
        his = history_bars(context.s2, 5, '1d', 'close')
        print(sorted(his.tolist()))
        print(sorted([24.1, 23.71, 23.82, 23.93, 23.66]))
        assert sorted(his.tolist()) == sorted([26.06, 26.13, 26.54, 26.6, 26.86])
'''

test_subscribe_code = '''
from rqalpha.api import subscribe
def init(context):
    context.f1 = 'AU88'
    context.amount = 1
    subscribe(context.f1)

def handle_bar(context, bar_dict):
    assert context.f1 in context.universe
'''

test_unsubscribe_code = '''
from rqalpha.api import subscribe, unsubscribe
def init(context):
    context.f1 = 'AU88'
    context.amount = 1
    subscribe(context.f1)
    unsubscribe(context.f1)

def handle_bar(context, bar_dict):
    assert context.f1 not in context.universe
'''

test_get_yield_curve_code = '''
from rqalpha.api import get_yield_curve
def init(context):
    pass

def handle_bar(context, bar_dict):
    df = get_yield_curve('20161101')
    assert df.iloc[0, 0] == 0.019923
    assert df.iloc[0, 6] == 0.021741
'''

test_history_bars_code = '''
from rqalpha.api import history_bars
def init(context):
    context.s1 = '000001.XSHE'
    pass

def handle_bar(context, bar_dict):
    return_list = history_bars(context.s1, 5, '1d', 'close')
    if str(context.now.date()) == '2016-12-29':
        assert return_list.tolist() == [9.08, 9.1199, 9.08, 9.06, 9.08]
'''

test_all_instruments_code = '''
from rqalpha.api import all_instruments
def init(context):
    pass

def handle_bar(context, bar_dict):
    df = all_instruments('FenjiA')
    df_to_assert = df.loc[df['order_book_id'] == '150247.XSHE']
    assert df_to_assert.iloc[0, 0] == 'CMAJ'
    assert df_to_assert.iloc[0, 7] == '工银中证传媒A'
    assert all_instruments().shape >= (8000, 4)
    assert all_instruments('CS').shape >= (3000, 16)
    assert all_instruments('ETF').shape >= (120, 9)
    assert all_instruments('LOF').shape >= (130, 9)
    assert all_instruments('FenjiMu').shape >= (10, 9)
    assert all_instruments('FenjiA').shape >= (120, 9)
    assert all_instruments('FenjiB').shape >= (140, 9)
    assert all_instruments('INDX').shape >= (500, 8)
    assert all_instruments('Future').shape >= (3500, 16)
'''

test_instruments_code = '''
from rqalpha.api import instruments
def init(context):
    context.s1 = '000001.XSHE'
    pass

def handle_bar(context, bar_dict):
    print('hello')
    ins = instruments(context.s1)
    assert ins.sector_code_name == '金融'
    assert ins.symbol == '平安银行'
    assert ins.order_book_id == context.s1
    assert ins.type == 'CS'
    print('world')
'''

test_sector_code = '''
from rqalpha.api import sector
def init(context):
    pass

def handle_bar(context, bar_dict):
    assert len(sector('金融')) >= 180
'''

test_industry_code = '''
from rqalpha.api import industry, instruments
def init(context):
    context.s1 = '000001.XSHE'
    context.s2 = '600340.XSHG'

def handle_bar(context, bar_dict):
    ins_1 = instruments(context.s1)
    ins_2 = instruments(context.s2)
    industry_list_1 = industry(ins_1.industry_name)
    industry_list_2 = industry(ins_2.industry_name)
    assert context.s1 in industry_list_1
    assert context.s2 in industry_list_2
'''

test_concept_code = '''
from rqalpha.api import concept, instruments
def init(context):
    context.s1 = '000002.XSHE'

def handle_bar(context, bar_dict):
    ins = instruments(context.s1)
    concept_list = concept(ins.concept_names[:4])  # 取 concept_names 的前4个字，即 context.s1 的第一个概念
    assert context.s1 in concept_list
    assert len(concept_list) >= 90
'''

test_get_trading_dates_code = '''
from rqalpha.api import get_trading_dates
import datetime
def init(context):
    pass

def handle_bar(context, bar_dict):
    trading_dates_list = get_trading_dates('2016-12-15', '2017-01-03')
    correct_dates_list = [datetime.date(2016, 12, 15), datetime.date(2016, 12, 16), datetime.date(2016, 12, 19),
                          datetime.date(2016, 12, 20), datetime.date(2016, 12, 21), datetime.date(2016, 12, 22),
                          datetime.date(2016, 12, 23), datetime.date(2016, 12, 26), datetime.date(2016, 12, 27),
                          datetime.date(2016, 12, 28), datetime.date(2016, 12, 29), datetime.date(2016, 12, 30),
                          datetime.date(2017, 1, 3)]
    assert sorted([item.strftime("%Y%m%d") for item in correct_dates_list]) == sorted([item.strftime("%Y%m%d") for item
                                                                                       in trading_dates_list])
'''

test_get_previous_trading_date_code = '''
from rqalpha.api import get_previous_trading_date
def init(context):
    pass

def handle_bar(context, bar_dict):
    assert str(get_previous_trading_date('2017-01-03').date()) == '2016-12-30'
    assert str(get_previous_trading_date('2016-01-03').date()) == '2015-12-31'
    assert str(get_previous_trading_date('2015-01-03').date()) == '2014-12-31'
    assert str(get_previous_trading_date('2014-01-03').date()) == '2014-01-02'
    assert str(get_previous_trading_date('2010-01-03').date()) == '2009-12-31'
    assert str(get_previous_trading_date('2009-01-03').date()) == '2008-12-31'
    assert str(get_previous_trading_date('2005-01-05').date()) == '2005-01-04'
'''

test_get_next_trading_date_code = '''

from rqalpha.api import get_next_trading_date
def init(context):
    pass

def handle_bar(context, bar_dict):
    assert str(get_next_trading_date('2017-01-03').date()) == '2017-01-04'
    assert str(get_next_trading_date('2007-01-03').date()) == '2007-01-04'

'''

test_get_dividend_code = '''
from rqalpha.api import get_dividend
import pandas
def init(context):
    pass

def handle_bar(context, bar_dict):
    df = get_dividend('000001.XSHE', start_date='20130104')
    df_to_assert = df.loc[df['book_closure_date'] == '	2013-06-19']
    assert df.shape >= (4, 5)
    assert df_to_assert.iloc[0, 1] == 0.9838
    assert df_to_assert.iloc[0, 3] == pandas.tslib.Timestamp('2013-06-20 00:00:00')
'''



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import namedtuple

import numpy as np

Rule = namedtuple('Rule', ['dtype', 'multiplier', 'round'])


class Converter(object):
    def __init__(self, rules):
        self._rules = rules

    def convert(self, name, data):
        try:
            r = self._rules[name]
        except KeyError:
            return data

        result = data * r.multiplier
        if r.round:
            result = np.round(result, r.round)

        return result

    def field_type(self, name, dt):
        try:
            return self._rules[name].dtype
        except KeyError:
            return dt

float64 = np.dtype('float64')

StockBarConverter = Converter({
    'open': Rule(float64, 1 / 10000.0, 4),
    'close': Rule(float64, 1 / 10000.0, 4),
    'high': Rule(float64, 1 / 10000.0, 4),
    'low': Rule(float64, 1 / 10000.0, 4),
    'limit_up': Rule(float64, 1/10000.0, 4),
    'limit_down': Rule(float64, 1/10000.0, 4),
})

FutureDayBarConverter = Converter({
    'open': Rule(float64, 1 / 10000.0, 2),
    'close': Rule(float64, 1 / 10000.0, 2),
    'high': Rule(float64, 1 / 10000.0, 2),
    'low': Rule(float64, 1 / 10000.0, 2),
    'limit_up': Rule(float64, 1 / 10000.0, 2),
    'limit_down': Rule(float64, 1 / 10000.0, 2),
    'basis_spread': Rule(float64, 1 / 10000.0, 4),
    'settlement': Rule(float64, 1 / 10000.0, 2),
    'prev_settlement': Rule(float64, 1 / 10000.0, 2),
})

FundDayBarConverter = Converter({
    'open': Rule(float64, 1 / 10000.0, 3),
    'close': Rule(float64, 1 / 10000.0, 3),
    'high': Rule(float64, 1 / 10000.0, 3),
    'low': Rule(float64, 1 / 10000.0, 3),
    'acc_net_value': Rule(float64, 1 / 10000.0, 4),
    'unit_net_value': Rule(float64, 1 / 10000.0, 4),
    'discount_rate': Rule(float64, 1 / 10000.0, 4),
    'limit_up': Rule(float64, 1/10000.0, 4),
    'limit_down': Rule(float64, 1/10000.0, 4),
})

IndexBarConverter = Converter({
    'open': Rule(float64, 1 / 10000.0, 2),
    'close': Rule(float64, 1 / 10000.0, 2),
    'high': Rule(float64, 1 / 10000.0, 2),
    'low': Rule(float64, 1 / 10000.0, 2),
})



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click
from rqalpha.__main__ import cli

__config__ = {
    "record": True,
    "output_file": None,
    "report_save_path": None,
    'plot': False,
    'plot_save_file': None,
}


def load_mod():
    from .mod import AnalyserMod
    return AnalyserMod()


"""
--report
--output-file

"""
cli.commands['run'].params.append(
    click.Option(
        ('--report', 'mod__sys_analyser__report_save_path'),
        type=click.Path(writable=True),
        help="[sys_analyser] save report"
    )
)
cli.commands['run'].params.append(
    click.Option(
        ('-o', '--output-file', 'mod__sys_analyser__output_file'),
        type=click.Path(writable=True),
        help="[sys_analyser] output result pickle file"
    )
)
cli.commands['run'].params.append(
    click.Option(
        ('-p', '--plot/--no-plot', 'mod__sys_analyser__plot'),
        default=None,
        help="[sys_analyser] plot result"
    )
)
cli.commands['run'].params.append(
    click.Option(
        ('--plot-save', 'mod__sys_analyser__plot_save_file'),
        default=None,
        help="[sys_analyser] save plot to file"
    )
)


@cli.command()
@click.argument('result_dict_file', type=click.Path(exists=True), required=True)
@click.option('--show/--hide', 'show', default=True)
@click.option('--plot-save', 'plot_save_file', default=None, type=click.Path(), help="save plot result to file")
def plot(result_dict_file, show, plot_save_file):
    """
    [sys_analyser] draw result DataFrame
    """
    import pandas as pd
    from .plot import plot_result

    result_dict = pd.read_pickle(result_dict_file)
    plot_result(result_dict, show, plot_save_file)


@cli.command()
@click.argument('result_pickle_file_path', type=click.Path(exists=True), required=True)
@click.argument('target_report_csv_path', type=click.Path(exists=True, writable=True), required=True)
def report(result_pickle_file_path, target_report_csv_path):
    """
    [sys_analyser] Generate report from backtest output file
    """
    import pandas as pd
    result_dict = pd.read_pickle(result_pickle_file_path)

    from .report import generate_report
    generate_report(result_dict, target_report_csv_path)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum
from collections import defaultdict


class Event(object):
    def __init__(self, event_type, **kwargs):
        self.event_type = event_type
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __repr__(self):
        return ' '.join('{}:{}'.format(k, v) for k, v in self.__dict__.items())


class EventBus(object):
    def __init__(self):
        self._listeners = defaultdict(list)

    def add_listener(self, event, listener):
        self._listeners[event].append(listener)

    def prepend_listener(self, event, listener):
        self._listeners[event].insert(0, listener)

    def publish_event(self, event):
        for l in self._listeners[event.event_type]:
            # 如果返回 True ，那么消息不再传递下去
            if l(event):
                break


class EVENT(Enum):
    # 系统初始化后触发
    # post_system_init()
    POST_SYSTEM_INIT = 'post_system_init'

    # 在实盘时，你可能需要在此事件后根据其他信息源对系统状态进行调整
    POST_SYSTEM_RESTORED = 'post_system_restored'

    # 策略执行完init函数后触发
    # post_user_init()
    POST_USER_INIT = 'post_user_init'
    # 策略证券池发生变化后触发
    # post_universe_changed(universe)
    POST_UNIVERSE_CHANGED = 'post_universe_changed'

    # 执行before_trading函数前触发
    # pre_before_trading()
    PRE_BEFORE_TRADING = 'pre_before_trading'
    # 该事件会触发策略的before_trading函数
    # before_trading()
    BEFORE_TRADING = 'before_trading'
    # 执行before_trading函数后触发
    # post_before_trading()
    POST_BEFORE_TRADING = 'post_before_trading'

    # 执行handle_bar函数前触发
    # pre_bar()
    PRE_BAR = 'pre_bar'
    # 该事件会触发策略的handle_bar函数
    # bar(bar_dict)
    BAR = 'bar'
    # 执行handle_bar函数后触发
    # post_bar()
    POST_BAR = 'post_bar'

    # 执行handle_tick前触发
    PRE_TICK = 'pre_tick'
    # 该事件会触发策略的handle_tick函数
    # tick(tick)
    TICK = 'tick'
    # 执行handle_tick后触发
    POST_TICK = 'post_tick'

    # 在scheduler执行前触发
    PRE_SCHEDULED = 'pre_scheduled'
    # 在scheduler执行后触发
    POST_SCHEDULED = 'post_scheduled'

    # 执行after_trading函数前触发
    # pre_after_trading()
    PRE_AFTER_TRADING = 'pre_after_trading'
    # 该事件会触发策略的after_trading函数
    # after_trading()
    AFTER_TRADING = 'after_trading'
    # 执行after_trading函数后触发
    # post_after_trading()
    POST_AFTER_TRADING = 'post_after_trading'

    # 结算前触发该事件
    # pre_settlement()
    PRE_SETTLEMENT = 'pre_settlement'
    # 触发结算事件
    # settlement()
    SETTLEMENT = 'settlement'
    # 结算后触发该事件
    # post_settlement()
    POST_SETTLEMENT = 'post_settlement'

    # 创建订单
    # order_pending_new(account, order)
    ORDER_PENDING_NEW = 'order_pending_new'
    # 创建订单成功
    # order_creation_pass(account, order)
    ORDER_CREATION_PASS = 'order_creation_pass'
    # 创建订单失败
    # order_creation_reject(account, order)
    ORDER_CREATION_REJECT = 'order_creation_reject'
    # 创建撤单
    # order_pending_cancel(account, order)
    ORDER_PENDING_CANCEL = 'order_pending_cancel'
    # 撤销订单成功
    # order_cancellation_pass(account, order)
    ORDER_CANCELLATION_PASS = 'order_cancellation_pass'
    # 撤销订单失败
    # order_cancellation_reject(account, order)
    ORDER_CANCELLATION_REJECT = 'order_cancellation_reject'
    # 订单状态更新
    # order_unsolicited_update(account, order)
    ORDER_UNSOLICITED_UPDATE = 'order_unsolicited_update'

    # 成交
    # trade(accout, trade)
    TRADE = 'trade'

    ON_LINE_PROFILER_RESULT = 'on_line_profiler_result'


def parse_event(event_str):
    return EVENT.__members__.get(event_str.upper(), None)



#!/usr/bin/env python
# encoding: utf-8
import inspect
import datetime


def test_order_shares():
    def init(context):
        pass

    def handle_bar(context, bar_dict):
        pass


def test_order_shares():
    from rqalpha.api import order_shares, get_order, SIDE, LimitOrder
    def init(context):
        context.order_count = 0
        context.s1 = "000001.XSHE"
        context.limitprice = 8.59
        context.amount = 10000

        pass

    def handle_bar(context, bar_dict):
        order_id = order_shares(context.s1, context.amount, style=LimitOrder(context.limitprice))
        order = get_order(order_id)
        order_side = SIDE.BUY if context.amount > 0 else SIDE.SELL
        assert order.side == order_side
        assert order.order_book_id == context.s1
        assert order.quantity == context.amount
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
        assert order.price == context.limitprice
test_order_shares_code_new = "".join(inspect.getsourcelines(test_order_shares)[0])


def test_order_lots():
    from rqalpha.api import order_lots, get_order, SIDE, LimitOrder
    def init(context):
        context.order_count = 0
        context.s1 = "000001.XSHE"
        context.limitprice = 8.59
        context.amount = 10000

        pass

    def handle_bar(context, bar_dict):
        order_id = order_lots(context.s1, 1, style=LimitOrder(context.limitprice))
        order = get_order(order_id)
        order_side = SIDE.BUY if context.amount > 0 else SIDE.SELL
        assert order.side == order_side
        assert order.order_book_id == context.s1
        assert order.quantity == 100
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
        assert order.price == context.limitprice
test_order_lots_code_new = "".join(inspect.getsourcelines(test_order_lots)[0])


def test_order_value():
    from rqalpha.api import order_value, get_order, SIDE, LimitOrder
    def init(context):
        context.order_count = 0
        context.s1 = "000001.XSHE"
        context.limitprice = 8.59
        context.amount = 10000

    def handle_bar(context, bar_dict):
        order_id = order_value(context.s1, 1000, style=LimitOrder(context.limitprice))
        order = get_order(order_id)
        order_side = SIDE.BUY if order.quantity > 0 else SIDE.SELL
        assert order.side == order_side
        assert order.order_book_id == context.s1
        assert order.quantity == 100
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
        assert order.price == context.limitprice
test_order_value_code_new = "".join(inspect.getsourcelines(test_order_value)[0])


def test_order_percent():
    from rqalpha.api import order_percent, get_order, SIDE, LimitOrder
    def init(context):
        context.order_count = 0
        context.s1 = "000001.XSHE"
        context.limitprice = 8.59
        context.amount = 10000

    def handle_bar(context, bar_dict):
        order_id = order_percent(context.s1, 0.0001, style=LimitOrder(context.limitprice))
        order = get_order(order_id)
        order_side = SIDE.BUY if order.quantity > 0 else SIDE.SELL
        assert order.side == order_side
        assert order.order_book_id == context.s1
        assert order.unfilled_quantity + order.filled_quantity == order.quantity
        assert order.price == context.limitprice
test_order_percent_code_new = "".join(inspect.getsourcelines(test_order_percent)[0])


def test_order_target_value():
    from rqalpha.api import order_target_percent, get_order, SIDE, LimitOrder
    def init(context):
        context.order_count = 0
        context.s1 = "000001.XSHE"
        context.limitprice = 8.59
        context.amount = 10000

    def handle_bar(context, bar_dict):
        order_id = order_target_percent(context.s1, 0.02, style=LimitOrder(context.limitprice))
        print("after: ", context.portfolio.cash)
        order = get_order(order_id)
        order_side = SIDE.BUY if order.quantity > 0 else SIDE.SELL
        assert order.side == order_side
        assert order.order_book_id == context.s1
        assert order.price == context.limitprice
test_order_target_value_code_new = "".join(inspect.getsourcelines(test_order_target_value)[0])



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime

import pandas as pd

from rqalpha.data.base_data_source import BaseDataSource
from rqalpha.environment import Environment
from rqalpha.model.snapshot import SnapshotObject


class DataSource(BaseDataSource):
    def __init__(self, path):
        super(DataSource, self).__init__(path)
        self._env = Environment.get_instance()
        self.realtime_quotes_df = pd.DataFrame()

    def get_bar(self, instrument, dt, frequency):
        # if frequency == '1d':
        #     return super(DataSource, self).get_bar(instrument, dt, frequency)

        # FIXME: 目前这样仅仅给撮合引擎用，不是定义的bar
        # FIXME: self.realtime_quotes_df 是从 event_source.py 那边「飞线」设置过来的
        bar = self.realtime_quotes_df.loc[instrument.order_book_id].to_dict()

        return bar

    def get_last_price(self, instrument, dt):
        return self.realtime_quotes_df.loc[instrument.order_book_id]['last']

    def current_snapshot(self, instrument, frequency, dt):
        snapshot_dict = self.realtime_quotes_df.loc[instrument.order_book_id].to_dict()
        snapshot_dict["last"] = snapshot_dict["price"]
        return SnapshotObject(instrument, snapshot_dict)

    def available_data_range(self, frequency):
        return datetime.date(2017, 1, 1), datetime.date.max
